@@ -6,8 +6,7 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-use super::{Error, Result, ToDbKey};
-use crate::types::utils::{deserialise, serialise};
+use super::{deserialise, serialise, Error, Result, ToDbKey};
 use serde::{de::DeserializeOwned, Serialize};
 use sled::{Db, Tree};
 use std::{fmt::Debug, marker::PhantomData};
@@ -82,7 +81,7 @@ mod test {
     async fn history() -> Result<()> {
         let id = xor_name::XorName::random();
         let tmp_dir = tempdir()?;
-        let db_dir = tmp_dir.into_path().join(Path::new(&"Token".to_string()));
+        let db_dir = tmp_dir.path().join(Path::new(&"Token".to_string()));
         let db = sled::open(db_dir).map_err(|error| {
             trace!("Sled Error: {:?}", error);
             Error::Sled(error)
